# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.drivers.sca_datashare")


class SCAdataShareSegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    message_date = HL7Date(7)
    message_type = HL7String((9, 1), default="ORM")
    message_code = HL7String((9, 2), default="001")
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.4")

    patients = HL7Foreign("PID", "header")


class SCAdataShareSegmentPID(Segment):
    patient_id = HL7String(4)
    last_name = HL7String((6, 1))
    first_name = HL7String((6, 2))
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    adress = HL7String(12)
    language = HL7String(16, default="FRA")

    visits = HL7Foreign("PV1", "patient")


class SCAdataShareSegmentPV1(Segment):
    doctor_id = HL7String((9, 1))
    doctor_first_name = HL7String((9, 2))
    doctor_last_name = HL7String((9, 3))

    common_order = HL7Foreign("ORC", "patient")
    orders = HL7Foreign("OBR", "patient")


class SCAdataShareSegmentORC(Segment):
    order_control = HL7String(2)
    specimen_id = HL7String(4)

    orders = HL7Foreign("OBR", "common_order")

    def to_string(self, separators=None):
        self.rang_segment = "NW"
        return super(SCAdataShareSegmentORC, self).to_string(separators=separators)


class SCAdataShareSegmentOBR(Segment):
    specimen_id = HL7String(3)
    service_identifier = HL7String((5, 1))
    service_name = HL7String((5, 2))
    observation_date = HL7Date(8)
    sample_received_date = HL7Date(15)

    results = HL7Foreign("OBX", "order")


class SCAdataShareSegmentOBX(Segment):
    value_type = HL7String(3)
    observation_identifier = HL7String(4)
    value = HL7String((6, 2))
    unit = HL7String(7)
    reference_ranges = HL7String(8)


class Driver(AstmMessageV2):
    SPEC = {"code": "sca_datashare", "label": "SCA dataShare"}
    SEGMENT_CLASS_MAP = {
        "MSH": SCAdataShareSegmentMSH,
        "PID": SCAdataShareSegmentPID,
        "PV1": SCAdataShareSegmentPV1,
        "OBR": SCAdataShareSegmentOBR,
        "ORC": SCAdataShareSegmentORC,
        "OBX": SCAdataShareSegmentOBX,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "sca_sending_application", label="Code émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "sca_sending_facility", label="Libellé émetteur", default="MCA"
        ),
        AstmMessageV2.Option(
            "sca_receiving_application", label="Code récepteur", default="SCA"
        ),
        AstmMessageV2.Option(
            "sca_receiving_facility", label="Libellé récepteur", default="SCA"
        ),
    ]

    def __init__(self, **kwargs):
        self.sca_sending_application = kwargs.pop("sca_sending_application", "MCA")
        self.sca_sending_facility = kwargs.pop("sca_sending_facility", "MCA")
        self.sca_receiving_application = kwargs.pop("sca_receiving_application", "SCA")
        self.sca_receiving_facility = kwargs.pop("sca_receiving_facility", "SCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return SCAdataShareSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        result_types = {"NM": "SN", "CE": "TX", "TX": "TX", "DT": "DT"}
        message = self.create(
            sending_application=self.sca_sending_application,
            sending_facility=self.sca_sending_facility,
            receiving_application=self.sca_receiving_application,
            receiving_facility=self.sca_receiving_facility,
            message_date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: "G"},
        ):
            adress = "{}, {} {}".format(
                patient.address, patient.postal_code, patient.city
            )
            seg_pid = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                adress=adress,
            )
            physician = [
                p for p in patient.physicians if p.type.lower() == "prescriber"
            ]
            seg_pv1 = seg_pid.visits.add(
                doctor_id=physician[0].code if physician else "",
                doctor_first_name=physician[0].first_name if physician else "",
                doctor_last_name=physician[0].last_name if physician else "",
            )
            for sample, action, priority in patient.samples:
                seg_orc = seg_pv1.common_order.add(specimen_id=sample.sample_id)
                for result in sample.results:
                    seg_obr = seg_orc.orders.add(
                        specimen_id=sample.sample_id,
                        service_identifier=patient.location,
                        service_name=patient.location_name,
                        observation_date=result.date,
                        sample_received_date=sample.request_date,
                    )
                    for key in result_types:
                        if result.type == key:
                            value_type = result_types.get(key)
                    seg_obr.results.add(
                        value_type=value_type,
                        observation_identifier=result.code,
                        value=result.value,
                        unit=result.unit,
                        reference_ranges=result.reference_ranges,
                    )

        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_pid in message.patients:
            for seg_pv1 in seg_pid.visits:
                for seg_obr in seg_pv1.orders:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.specimen_id, type="patient"
                    )
                    date = seg_obr.observation_date
                    for seg_obx in seg_obr.results:
                        sample.results.add(
                            code=seg_obx.observation_identifier,
                            value=seg_obx.value,
                            unit=seg_obx.unit,
                            date=date,
                        )
        return worklist
